Package: SVAlignR
Version: 0.9.2
Date: 2025-09-20
Title: Recovering Structure of Long Molecules from Structural Variation
        Data
Authors@R: person(given = c("Kevin", "R."), family = "Coombes",
                  role = c("aut", "cre"), email = "krc@silicovore.com")
Description: Implements a method to combine multiple levels of multiple
  sequence alignment to uncover the structure of complex DNA rearrangements.
Depends: R (>= 4.4)
Imports: methods, graphics, grDevices, oompaBase, Biostrings,
        NameNeedle, dendextend, ape, stringr, igraph, Polychrome,
        colorspace
Suggests: msa, viridisLite, R.rsp, knitr, rmarkdown
VignetteBuilder: R.rsp
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
NeedsCompilation: no
Packaged: 2025-09-20 12:54:30 UTC; KRC
Author: Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: CRAN
Date/Publication: 2025-09-24 08:40:02 UTC
Built: R 4.6.0; ; 2026-04-28 21:54:44 UTC; unix

Package: SVAlignR
Version: 0.10.4
Date: 2026-06-18
Title: Recovering Structure of Long Molecules from Structural Variation
        Data
Authors@R: person(given = c("Kevin", "R."), family = "Coombes",
                  role = c("aut", "cre"), email = "krc@silicovore.com")
Description: Implements a method to combine multiple levels of multiple
  sequence alignment to uncover the structure of complex DNA rearrangements.
Depends: R (>= 4.4)
Imports: methods, graphics, grDevices, oompaBase, Biostrings,
        NameNeedle, dendextend, ape, stringr, igraph, Polychrome,
        colorspace
Suggests: msa, viridisLite, R.rsp, knitr, rmarkdown
VignetteBuilder: R.rsp
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
Author: Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: clonefinder
Repository/R-Forge/Revision: 195
Repository/R-Forge/DateTimeStamp: 2026-06-18 20:20:13
Date/Publication: 2026-06-18 20:20:13
NeedsCompilation: no
Packaged: 2026-06-22 10:44:09 UTC; rforge

###
### NAMESPACE
###

import("methods")
importMethodsFrom("oompaBase", "plot", "image")

importFrom("graphics", "image", "mtext", "text", "abline", "lines")
importFrom("grDevices", "topo.colors")
importFrom("stats", "aggregate",
           "hclust", "as.dendrogram",
           "as.dist", "cutree", "heatmap")
importFrom("utils", "adist", "write.csv")

### Computational models
importFrom("NameNeedle", "needleScores", "defaultNeedleParams")
#importFrom("msa", "msa", "msaConsensusSequence", "msaClustalW")
importFrom("Biostrings", "AAStringSet")

### graphing
importFrom("dendextend", "set", "%>%")
importFrom("ape", "as.phylo")
importFrom("Polychrome", "createPalette", "dark.colors")
importFrom("igraph", "graph_from_edgelist", "graph_from_adjacency_matrix",
           "set_vertex_attr", "vertex_attr", "set_edge_attr", "edge_attr")
importFrom("stringr", "str_count", "str_locate")
importFrom("colorspace", "hex2RGB")

### Exports
exportClass("Cipher")
export("Cipher", "encode", "decode")

### SequenceClusters
exportClass("SequenceCluster")
export("SequenceCluster", "updateClusters", "heat")
export("makeSubsMatrix", "alignCluster", "alignAllClusters")

exportClass("AlignedCluster")

exportMethods("plot", "image")

### StringGraphs
exportClass("StringGraph")
export("MotifGraph", "DecompositionGraph", "exportSG")

## KMers
export("makeWords", "countWords", "plotWords")

### Breakpoints
exportClass("Breakpoints")
export("Breakpoints")

### De Bruijn Graphs
exportClass("DeBruijn")
export("deBruijn")

## Evidence for putative cycles
export("getEvidence")

library(SVAlignR)
data(longreads)
seqs <- longreads$connection[1:15]
pad <- c(rep("0", 9), rep("", 6))
names(seqs) <- paste("LR", pad, 1:length(seqs), sep = "")
seqs <- seqs[!duplicated(seqs)]

try( makeSubsMatrix(match = "13") )
try( makeSubsMatrix(mismatch = "13") )

mysub <- makeSubsMatrix(match = 2, mismatch = -6)

## test the 'alignCluster' method
try( alignCluster(13) )

ab <- alignCluster(seqs, mysub)
image(ab)

Vexpand <- function(cycles, seqs, alfa) {
  countVs <- function(cycs) {
    ss <- strsplit(cycs, 'v')
    sapply(ss, length) - 1
  }
  if( any(countVs(cycles) > 1) ) {
    stop("Input cycle is not a simple path!")
  }
  repVs <- function(seqs, V) {
    K <- 0
    count <- 1
    while(count > 0) {
      K <- K + 1
      vrep <- paste(rep(V, K), collapse = "-")
      count <- sum(grepl(vrep, seqs))
    }
    K - 1
  }
  # find repeated breakpoints
  enc <- encode(alfa, seqs)
  matches <- gregexpr("(.)\\1", enc)
  repeated_chars <- unique(unlist(regmatches(enc, matches)))
  stutter <- decode(alfa, substring(repeated_chars, 1, 1))
  for (V in stutter) {
    N <- repVs(seqs, V)
    if (N > 1) {
      hasV <- seqs[grep(V, cycles)]
      sap <- sapply(2:N, function(J, cycs) {
        vrep <- paste(rep(V, J), collapse = "-")
        sub(V, vrep, cycs)
      }, cycs = cycles)
      cycles <- c(cycles, as.vector(sap))
    }
  }
  longcM1 <- cycles[!duplicated(cycles)]
  longcM1
}

bothWays <- function(xc, seqs) {
  revert <- function(cycle) {
    X <- strsplit(cycle, "-")[[1]]
    paste(rev(X), collapse = "-")
  }
  doubles <- paste0(xc, "-", xc)
  support <- sapply(seqs, grep, x = doubles)
  RE <- sapply(seqs, revert)
  troppus <- sapply(RE, grep, x = doubles)
  comb <- lapply(1:length(support), function(J) {
    unique(c(support[[J]], troppus[[J]]))
  })
  names(comb) <- names(support)
  list(comb=comb, doubles = doubles)
}

getSupport <- function(comb, clues) {
  L <- sapply(comb, length)
  SP <- data.frame(seq=NA, cyc = NA)[-1,]
  for (J in 1:length(comb)) {
    if (L[J] > 0) {
      tp <- data.frame(seq = names(comb)[J], cyc = comb[J])
      colnames(tp) <- c("seq", "cyc")
      SP <- rbind(SP, tp)
    }
  }
  SP$weight <- clues@weights[SP$seq]
  SP
}

getCoverage <- function(xc, rawSP, seqs, doubles, alfa) {
  cvg <- lapply(1:length(doubles), function(K) {
    fd <- seqs[rawSP$seq[rawSP$cyc == K]]
    gp <- encode(alfa, fd)
    DD <- encode(alfa, doubles[[K]])
    M <- rep(0, length(strsplit(xc[K], "-")[[1]]))
    strt <- sapply(gp, function(S) str_locate(DD, S))
    if (is.null(dim(strt))) return(list(N = 0, RDS = "", TOT = 0, COV = M, WCOV = M))
    M2 <- c(M, M)
    W2 <- M2
    rds <- NULL
    for (J in 1:ncol(strt)) {
      if (!is.na(strt[1,J])) {
        A <- strt[1,J]
        B <- strt[2,J]
        M2[A:B] <- M2[A:B] + 1
        W2[A:B] <- W2[A:B] + rawSP$weight[J]
        rds <- c(rds, colnames(strt)[J])
      }
    }
    half <- length(M)
    M <- M2[1:half] + M2[half + (1:half)]
    W <- W2[1:half] + W2[half + (1:half)]
    list(N = ncol(strt), RDS = paste(rds, collapse = ";"),
         TOT = min(M), COV=M, WCOV = W)
  })
  cvg
}

showCoverage <- function(ID, evidence, alfa, colsams, locsams = NULL) {
  CC <- evidence$coverage[[ID]]
  SQ <- evidence$vspan[ID]
  N <- length(sq <- strsplit(SQ, "-")[[1]])
  lrnames <- strsplit(CC$RDS, ";")[[1]]
  if (!is.null(locsams)) {
    lrnames <- lrnames[order(locsams[lrnames])]
  }
  L <- length(lrnames)
  if (missing(colsams)) {
    colsams <- rep("grey", L)
    names(colsams) <- lrnames
  }
  target <- encode(alfa, SQ)
  target <- paste0(target, target)
  plot(c(1/2,N+1/2), c(1,(1+L)),  type = "n", xaxt = "n", yaxt = "n",
       xlab = "breakpoints", ylab = "long reads",
       main = paste("Putative Cycle", ID))
  mtext(lrnames, side = 2, at = 1:L, las = 2, cex = 0.5)
  text(1:N, L+1, sq)
  for (J in 1:L) {
    lr <- lrnames[J]
    clr <- colsams[lr]
    S <- encode(alfa, sqn[lr])
    finder <- str_locate(target, S)
    strt <- finder[1, 1]
    end <- finder[1, 2]
    if (end <= N) {
      lines(c(strt-0.5, end+0.5),c(J, J),  col = clr, lwd = 5 - round(log10(L)))
    } else {
      lines(c(strt-0.5, N+0.5),c(J, J),  col = clr, lwd = 5)
      lines(c(0.5, end-N+0.5),c(J, J),  col = clr, lwd = 5)
    }
    lines(c(strt-0.5, strt-0.5), c(J-0.4, J+0.4), col = "black", lwd=4)
  }
}

getEvidence <- function(cycles, seqs, alfa, totalOnly = TRUE) {
  xcM1 <- Vexpand(cycles, seqs, alfa)
  BW <- bothWays(xcM1, seqs)
  SP <- getSupport(BW$comb, seqclust)
  coverage <- getCoverage(xcM1, SP, seqs, BW$doubles, alfa)
  wmin <- sapply(coverage, function(X) min(X$WCOV))
  drat <- data.frame(N = unlist(lapply(coverage, function(X) X["N"])),
                     RDS = unlist(lapply(coverage, function(X) X["RDS"])),
                     TOT = unlist(lapply(coverage, function(X) X["TOT"])),
                     WTOT = wmin,
                     cycle = xcM1)
  if (totalOnly) {
    drat <- drat[drat$TOT > 0,]
  }
  list(coverage = coverage, circles = drat, vspan = xcM1)
}

### Working With Ciphers

setClass("Cipher",
         slots = c(
           forward = "character",
           reverse = "character",
           bytes = "numeric")
         )

Cipher <- function(sampleText, split = "-", extras = c("-" = ":", "?" = "?")) {
  nb <- 1
  shatter <- strsplit(sampleText, split) # decompose into lists of symbols
  U <- sort(unique(unlist(shatter))) # find the unique symbols
  base <- c(LETTERS[-c(10,24)], letters, 1:9, 0,
            "!", "@", "#", "$", "%", "^", "&", "|", "_", ";", "/", ",")
  if (length(U) > length(base)) {
    warning("Input has more than ", length(base), " letters. ",
            "Using two-byte codes.\n")
    nb <- 2
    multi <- 1 + trunc(length(U)/length(base))
    base <-paste0(rep(LETTERS[1:multi], each = length(base)),
                    rep(base, times = multi))
  }
  forward  <- base[1:length(U)]
  names(forward) <- U
  if (is.null(names(extras)) | any(names(extras) == "")) {
    stop("All elements of 'extras' must have names.\n")
  }
  if (any(duplicated(c(names(forward), extras)))) {
    stop("Values of 'extras' cannot match letter in the 'forward' cipher.\n")
  }
  reverse <- names(forward)
  names(reverse) <- forward
  reverse <- c(reverse, extras)
  if (any(duplicated(names(reverse)))) {
    stop("Names of 'extras' cannot include symbols in 'sampleText'.\n")
  }
  new("Cipher", forward = forward, reverse = reverse, bytes = nb)
}

.xlate <- function(txt, language, split, collapse, nb = 1) {
  txt <- strsplit(txt, split)
  if (nb > 1) {
    txt <- lapply(txt, function(TX) {
      tmat <- matrix(TX, ncol = nb, byrow = TRUE)
      apply(tmat, 1, paste, collapse = "")
    })
  }
  temp <- unique(unlist(txt))
  fail <- !(temp %in% names(language))
  if (any(fail)) {
    stop("Text includes unknown characters: ", temp[fail])
  }
  sapply(txt, function(X) {
    paste(language[X], collapse = collapse)
  })
}

encode <- function(cipher, text) {
  .xlate(text, cipher@forward, "\\-", "", 1)
}

decode <- function(cipher, text) {
  .xlate(text, cipher@reverse, "", "-", cipher@bytes)
}

setClass("Breakpoints",
         slots = c(relLocation = "numeric",
                   labels = "character",
                   ypos = "numeric",
                   spread = "numeric",
                   id = "character"))
## Decide if we wnat to keep enough stuff so we can create
## circos plots of all translocations.

Breakpoints <- function(working) {
  A <- data.frame(working[, 1:3], side = "left")
  B <- data.frame(working[, c(1, 5:6)], side = "right")
  colnames(A) <- colnames(B) <- c("id", "chrom", "base", "side")
  R <- rbind(A, B)
## need to get Keiko to use chromosome names and not new segment names
  R <- R[order(R$chrom, R$base),]
  U <- unique(R$chrom)
  pick <- sapply(U, function(u) R$chrom == u) 
  mini <- aggregate(R$base, list(R$chrom), min)$x
  maxi <- aggregate(R$base, list(R$chrom), max)$x
  delta <- maxi - mini
  names(maxi) <- names(mini) <- names(delta) <- U
  unitcoords <- apply(R, 1, function(arow) {
    ch <- as.character(arow[2])
    (as.numeric(arow[3]) - mini[ch])/delta[ch]
  })

  ct <- as.numeric(factor(R$chrom))
  spread <- c(0.4, 0, -0.4, 0.1, -0.3, 0.2, -0.2 ,0.3, -0.1)
  spread <- rep(spread, times = 1 + trunc(length(ct/9)))[1:length(ct)]
  new("Breakpoints",
      relLocation = unitcoords,
      labels = U,
      ypos = ct,
      spread = spread,
      id = R$id)
}

setMethod("plot", c("Breakpoints", "missing"), function(x, y, colset, ...) {
  N <- length(x@labels)
  if(missing(colset)) colset <- dark.colors(N)
  plot(x@relLocation, x@ypos + x@spread,
       type = "n", ylim = c(0.5, N + 0.5), yaxt = "n",
       xlab = "Relative Base Posiiton", ylab = "Chromosome")
  mtext(x@labels, side = 2, line = 1, at = 1:N, las = 3)
  text(x@relLocation, x@ypos + x@spread, font = 2,
       x@id, srt=90, col = colset[x@ypos])
  abline(h = (1:N) +0.5, col = "gray")
  invisible(x)
})

### Initial Pass: Clustering Sequences

setOldClass("dist")
setOldClass("hclust")

setClass("SequenceCluster",
         slots = c(
           method = "character",
           rawSequences = "character",
           weights = "numeric",
           distance = "dist",
           hc = "hclust",
           NC = "numeric",
           clusters = "numeric")
         )

SequenceCluster <- function(rawseq, method = c("needelman", "levenshtein"), NC = 5) {
  method <- match.arg(method)

  doNeedelman <- function(pseudo) {
    mypar <- defaultNeedleParams
    mypar$MATCH <- 2
    mypar$MISMATCH <- -6
    mypar$GAP <- -2
    mat <- matrix(NA, nrow = length(pseudo), ncol = length(pseudo))
    rownames(mat) <- colnames(mat) <- names(pseudo)
    for (i in 1:length(pseudo)) {
      scores <- needleScores(pseudo[i], pseudo, mypar)
      mat[i, ] <- (scores - min(scores))/(max(scores) - min(scores))
    }
    as.dist(1 - (mat + t(mat))/2)
  }

  doLevenshtein <- function(pseudo) {
    mat <- matrix(NA, nrow = length(pseudo), ncol = length(pseudo))
    rownames(mat) <- colnames(mat) <- names(pseudo)
    for (i in 1:length(pseudo)) {
      mat[i,] <-  adist(pseudo[i], pseudo)
    }
    as.dist(mat)
  }
  
  ## Make sure everything is named
  if (is.null(names(rawseq))) {
    warning("Adding names to input sequences.")
    L <- length(rawseq)
    nzero <- trunc(log10(1:L))
    pad <- sapply(max(nzero) - nzero, function(n) paste(rep("0", n), collapse = ""))
    names(rawseq) <- paste("SEQ", pad, 1:L, sep = "")
  }

  ## dedup
  if (any(dup <- duplicated(rawseq))) {
    warning("Removing ", sum(dup), " duplicated sequences.\n")
    dedup <- rawseq[!dup]
    weights <- sapply(dedup, function(lr) sum(rawseq == lr))
    rawseq <- dedup
  } else  {
    weights <- rep(1, length(rawseq))
  }
  names(weights) <- names(rawseq)

  ## encode the input sequences
  alphabet <- Cipher(rawseq)
  encoded <- encode(alphabet, rawseq)
  ## compute distance matrix as 'dist' object
  symm <- switch(method,
                 needelman = doNeedelman(encoded),
                 levenshtein = doLevenshtein(encoded),
                 stop("Method '", method, "' does not match any known methods." )
                 )
  hc <- hclust(symm, "ward.D2")
  clust <- cutree(hc, k = NC)
  new("SequenceCluster",
      method = method,
      rawSequences = rawseq,
      weights = weights,
      distance = symm,
      hc = hc,
      NC = NC,
      clusters = clust)
}

updateClusters <- function(sc, NC) {
  sc@NC <- NC
  sc@clusters <- cutree(sc@hc, k = NC)
  sc
}


myColorSet <- c("cornflowerblue", "hotpink2", "green4", "red",
                "darkorchid1", "gray16", "darkgoldenrod", "mediumorchid4",
                "coral1", "blue", "lightseagreen", "rosybrown",
                "darkolivegreen4", "skyblue4", "tomato4", "mediumpurple2",
                "hotpink4", "orchid", "tan4", "midnightblue",
                "firebrick", "maroon1", "deeppink",  "blue3")

setMethod("plot", signature("SequenceCluster", "missing"),
function(x, type = "rooted", main = "Colored Clusters", ...) {
  plotRooted <- function(x, NC = x@NC, ...) {
    ##    cat("rooted\n", file = stderr())
    groups <- unique(cutree(x@hc, k = NC)[x@hc$order])
    colsch <- myColorSet[groups]
    hcd <- as.dendrogram(x@hc)
    hcd %>% set("labels_col", value = colsch, k = NC) %>% 
      set("branches_k_color", value = colsch, k = NC) %>%
      set("branches_lwd", 2) %>% 
      set("labels_cex", 0.7) %>%
      plot(main = main)
  }
  plotClipped <- function(x, NC, ...) {
 ##   cat("clipped\n", file = stderr())
    groups <- unique(cutree(x@hc, k = NC)[x@hc$order])
    colsch <- myColorSet[groups]
    mycut <- mean(rev(x@hc$height)[NC + -1:0])
    hcd <- as.dendrogram(x@hc)
    dend2 <- cut(hcd, h = mycut)
    dend2$upper %>% set("labels_col", value = colsch, k = NC) %>% 
      set("branches_k_color", value = colsch, k = NC) %>%
      set("branches_lwd", 2) %>% 
      set("labels_cex", 1.3) %>%
      plot(main = main)
  }
  plotUnrooted <- function(x, NC, ...) {
 ##   cat("unrooted\n", file = stderr())
    typer <- cutree(x@hc, k = NC)
    plot(as.phylo(x@hc), type = "unrooted",
         edge.color = "steelblue", edge.width = 2,
         tip.color = myColorSet[typer], cex = 0.7)
  }

  NC <- x@NC
  types <- c("rooted", "clipped", "unrooted")
  type <- match.arg(type, types)
##  cat("Calling", type, "\n")
  val <- switch(type,
                rooted = plotRooted(x, NC, ...),
                clipped = plotClipped(x, NC, ...),
                unrooted = plotUnrooted(x, NC, ...),
                warning("Invalid type.")
                )
  invisible(x)
})

heat <- function(x, ...) {
  S <- as.matrix(x@distance)
  hc <- x@hc
  typer <- cutree(hc, k = x@NC)
  if(requireNamespace("viridisLite")) {
    colscheme <- viridisLite::viridis(64)
  } else {
    colscheme <- topo.colors(64)
  }
  heatmap(S, ColSideColors = myColorSet[typer],
        Rowv = as.dendrogram(hc),
        Colv = as.dendrogram(hc), col = colscheme, scale = "none")
}

makeSubsMatrix <- function(match = 5, mismatch = -2) {
  if (!is.numeric(match) | !is.numeric(mismatch)) {
    stop("Both parameters must be numeric.\n")
  }
  if (length(match) != 1 | length(mismatch) != 1) {
    stop("Both parametersm must be of length one.\n")
  }
  lab <- c(LETTERS[c(-15, -21)], "*" )
  MYSUB <- matrix(mismatch, 25, 25, dimnames = list(lab, lab))
  diag(MYSUB) <- match
  MYSUB
}

### Inputs:
### 'sequences' is a character vector representing sequences
### in an arbitrary alphabet with at most 25 distinct characters
###
### Values = a list with three components
###    babel = original sequences translated into an AAStringSet
###    aligned = results of an alignment algorithm
###    alignedOriginal = results translated back to starting alphabet
align <- function(sequences, mysub = NULL, gapO = 10, gapE = 0.2) {
  if (!requireNamespace("msa", quietly = TRUE)) {
    stop("The `align` function will only work if the 'msa' package is ",
         "already installed.\n")
  }
  if (is.null(mysub)) mysub <- makeSubsMatrix()
  U2 <- sort(unique(unlist(strsplit(sequences, ""))))
  if (length(U2) > 25) {
    stop("Cannot handle an alphabet with more than 25 letter!")
  }
  ## Translate from the starting alphabet to amino acids
  protein <- Cipher(sequences, split = "", extras = c("-" = "-", "?" = "?"))
  tempaa <- .xlate(sequences, protein@forward, "", "")
  babel <- AAStringSet(tempaa) # pretend we have proteins
  ## Align the sequences with a custom matrix
  aligned <- msa::msa(babel, method = "ClustalW",
                 substitutionMatrix = mysub, gapOpening = gapO, gapExtension = gapE)
  ## Translate back to the original alphabet
  reverse <- .xlate(as.character(aligned), protein@reverse, "", "")
  xcons  <- .xlate(msa::msaConsensusSequence(aligned), protein@reverse, "", "")

  list(babel = babel,
       aligned = aligned,
       alignedOriginal = reverse, 
       cons = xcons)
}

setClass("AlignedCluster",
         slots = c(
           alignment = "matrix",
           weights = "numeric",
           consensus = "character")
         )

alignCluster <- function(sequences, mysub = NULL, gapO = 10, gapE = 0.2) {
  ## assume we have names on the individual sequences
  if (is.null(mysub)) mysub <- makeSubsMatrix()
  seqs <- sequences[!duplicated(sequences)]  # dedup
  alfa <- Cipher(seqs)
  enc  <- encode(alfa, seqs)                 # encode as though amino acids
  sva  <- try(align(enc, mysub, gapO, gapE), silent = TRUE) # align using ClustalW
  if (inherits(sva, "try-error")) {
    warning("Alphabet exceeds 25 characters.")
    return(NULL)
  }
  cons <- decode(alfa, sva$cons)             # decode the consensus sequence
  back <- decode(alfa, sva$alignedOriginal)  # decode the aligned sequences
  rack <- strsplit(back, "-")
  new("AlignedCluster",
      alignment = as.matrix(as.data.frame( rack )),
      weights = integer(0),
      consensus = cons)
}


alignAllClusters <- function(sc, mysub = NULL, gapO = 10, gapE = 0.2) {
  if (!inherits(sc, "SequenceCluster")) {
    stop("Bad input, sc = ", class(sc), "\n")
  }
  if (is.null(mysub)) mysub <- makeSubsMatrix()
  NC <- sc@NC
  result <- lapply(1:NC, function(K) {
    ab <- alignCluster(sc@rawSequences[sc@clusters == K],
                       mysub = mysub, gapO = gapO, gapE = gapE)
    if (!is.null(ab)) {
      ab@weights <- sc@weights[colnames(ab@alignment)]
    }
    ab
  })
  nzero <- trunc(log10(1:NC))
  pad <- sapply(max(nzero) - nzero, function(n) paste(rep("0", n), collapse = ""))
  names(result) <- paste("B", pad, 1:NC, sep = "")
  result
}


setMethod("image", "AlignedCluster",  function(x, counts = TRUE, col = "black", cex = 1, main = "", ...) {
  meet <- x@alignment
  wt <- x@weights
  cons <- unlist(strsplit(x@consensus, "-"))
  matr <- matrix(1, nrow = nrow(meet), ncol = ncol(meet))
  matr[cons == "?", ] <- 2
  matr[!(cons %in% c("?", ":"))] <- 3
  image(1:nrow(meet), 1:ncol(meet),
        matr, col = c("gray89", "navajowhite", "lightsteelblue1"),
        yaxt = "n", xlab = "Position", ylab = "", zlim = c(1,3))
  for (i in 1:nrow(meet)) {
    text(i, 1:ncol(meet), meet[i,], cex = cex)
  }
  tag <- apply(meet, 1, function(x) {
    x <- x[!(x %in% c(":", "x"))]
    val <- names(which.max(table(x)))
    if(is.null(val)) val <- ":"
    val
  })
  tagcex <- ifelse(nrow(meet) > 20, 0.8, 1)
  mtext(tag, side = 3, line = 1, at = 1:nrow(matr), col = col, cex = tagcex)
  txtcex <- ifelse(ncol(meet) > 15, 
                   ifelse(ncol(meet) > 22, 0.5, 0.7), 1)
  mtext(colnames(meet), side = 2, las = 2, at = 1:ncol(meet), line = 0.5,
        col = col, cex = txtcex)
  if (counts & length(wt) == ncol(meet)) {
    mtext(paste("n=", wt, sep = ""), side = 4, line = 1/2, at = 1:length(wt), las=2)
  }
  invisible(x)
})

library(SVAlignR)
data(longreads)
N <- 32
sequences <- longreads$connection[1:N]

# specify Levenshtein (should get two warnings)
sc1 <- SequenceCluster(sequences, method = "levenshtein", NC = 4)
plot(sc1@hc)

# after assigning names, should get one warning
names(sequences) <- rownames(longreads)[1:N]
sc1 <- SequenceCluster(sequences, method = "levenshtein", NC = 4)

# after removing duplicates, should get no warnings
sequences <- sequences[!duplicated(sequences)]
sc1 <- SequenceCluster(sequences, method = "levenshtein", NC = 4)

# default Needelman-Wunsch
sc <- SequenceCluster(sequences, NC = 4)
plot(sc)
# Other plot forms
plot(sc, type = "clipped")
plot(sc, type = "unrooted")


# make sure unknown methods fail.
try( sc <- SequenceCluster(sequences, method = "clustal") )
try( sc3 <- SequenceCluster(sequences, method = "leven") ) # Now OK

# same with unknown plot types
try( plot(sc, type = "doodle") )

#### update cluster number
sc <- updateClusters(sc, NC = 3)
plot(sc, type = "unrooted")

library(SVAlignR)

motif <- "0-50-74-0-50-74-25-26-35"
alfa <- Cipher(motif)
alfa

en <-encode(alfa, motif)
en

de <- decode(alfa, en)
de
de == motif

## bad Cipher inputs
try( Cipher(13) )
try( Cipher(motif, extras = 13) )
try( Cipher(motif, extras = c(z = "zero", 13)) )
try( Cipher(motif, extras = c(A = "zero")) )
try( Cipher(motif, extras = c("x" = 26)) )

odd <- c(motif, "0-0-50-74-61")
try( encode(alfa, odd) )
try( encode(odd, alfa) )

## Longer alphabets
nchar(IT <- intToUtf8(L <- c(38:44, 46:57, 59:62, 64:126)))
## omit hyphen, colon, and question
JT <- paste(strsplit(IT, "|")[[1]], collapse = "-")
gamma <- Cipher(JT)

b2 <- encode(gamma, JT)
decode(gamma, b2) == JT

library(SVAlignR)
data(longreads)
seqs <- longreads$connection[1:15]
pad <- c(rep("0", 9), rep("", 6))
names(seqs) <- paste("LR", pad, 1:length(seqs), sep = "")
seqs <- seqs[!duplicated(seqs)]

alfa <- Cipher(seqs)
seqclust <- SequenceCluster(seqs, NC = 5)
plot(seqclust)
coded <- encode(alfa, seqclust@rawSequences)
kmers <- lapply(1:11, function(N) {countWords(coded, N)})
#motifNodes <- lapply(kmers, function(x) x[names(x) %in% names(motifs)])
#save(alfa, motifs, kmers, motifNodes, decomp,
#     file = file.path(paths$scratch, "motifNodes.Rda"))

## opstrings is a character vector containing already encoded sequences
## K is the length of the words to be listed
## nb is the number of bytes used to encode each character
dekomp <- function(opstrings, K, nb = 1) {
  temp <- sapply(opstrings, function(use, K) {
    ## cat("Use:", class(use), " ", length(use), "\n", file = stderr())
    ### use is always a single character string
    pop <- strsplit(use, "")[[1]]
    ## cat("Pop:", class(pop), " ", length(use), "\n", file = stderr())
    ## cat("Sent:", pop, "\n")
    ### pop is also a single character string
    if (nb > 1) {
      tmat <- matrix(pop, ncol = nb, byrow = TRUE)
      pop <- apply(tmat, 1, paste, collapse = "")
      ## cat("After Pop:", class(pop), " ", length(use), "\n", file = stderr())
      ## cat("Got: ", pop, "\n", file = stderr())
    }
    L <- length(pop)
    if (L < K) return(NULL)
    if (L == K) {
      val <- 1
      names(val) <- use
      return(use)
    }
    sapply(0:(L-K), function(S) paste0(pop[S + (1:K)], collapse=""))
  }, K = K)
  temp
}

makeWords <- function(opstrings, K, nb = 1) {
  temp <- dekomp(opstrings, K, nb)
  table(unlist(temp)) # this is the slow step. look here if you need to speed it up
}

## opstrings is a character vector containing already encoded sequences
countWords <- function(opstrings, K, alpha = NULL) {
  nb <- ifelse(is.null(alpha), 1, alpha@bytes)
  m <- makeWords(opstrings, K, nb)
  if (!is.null(alpha)) {
    names(m) <- decode(alpha, names(m))
  }
  m
}

## 'm' is a list consisting of the ouput of countWords for different integers
## 'k' is the word length
plotWords <- function(K, m) {
  V <- as.vector(m[[K]])
  N <- rownames(as.matrix(m[[K]]))
  oo <- order(V)
  V <- V[oo]
  N <- N[oo]
  L <- length(V)
  if (L > 50) {
    keep <- (L-50):L
    V <- V[keep]
    N <- N[keep]
    L <- length(V)
  }
  plot(1:L, V, type = "n", ylim = c(-5, 5 + max(V)),
       xlab = "Index", ylab = "Count", main = paste(K, "mers", sep = "-"))
  text(1:L, V, N, srt = 90)
  invisible(V)
}

setClass("DeBruijn",
         slots = c("G" = "igraph",
                   "adjmat" = "matrix",
                   "motifs" = "table"))

## rawseq are the raw sequences, typically separated by hyphens since
## they are represented as multicharacter symbols (often chromosome number
## plus an arbitrary alphabetic character).
##
## M = length of motifs to be used in the DeBruijn graph.
##
deBruijn <- function(rawseq, M) {
  ## Start by re-encoding in a single character alphabet.
  bethe <- Cipher(rawseq)
  rs <- encode(bethe, rawseq)
  ## Get a list of motifs (character strings) of length M
  ## present in the data. Note that the `countWordss` function
  ## wants to convert the element names back to the original alphabnet
  motifs <- countWords(rs, M, bethe)
  ## decompose each long-read breakpoint sequences (LRBPS) as a
  ## sequence of overlapping M-mers. Uses this function:
  nb <- bethe@bytes
  ## Don't mess with really small sequences
  rs6 <- rs[nchar(rs) > M*nb]
  ## actually decompose everything that is decomposable
  layers <- sapply(rs6, function(sqn) {
    dek <- decode(bethe, dekomp(sqn, M, nb))
    sapply(dek, function(src) {
      which(names(motifs) == src)
    })
  })
  ## Create a (directed) adjacency matrix from the decompositions
  adjmat <- matrix(0, nrow = length(motifs), ncol = length(motifs))
  rownames(adjmat) <- colnames(adjmat) <- names(motifs)
  for (LRval in names(layers)) {
    LYR <- layers[[LRval]]
    for (J in 1:(length(LYR) - 1)) {
      a <- names(LYR)[J]
      b <- names(LYR)[J + 1]
      DBG <- adjmat[a, b]
      adjmat[a, b] <- DBG + 1
#      cat(a, "\t", b, "\t", DBG, "\t", adjmat[a, b],  "\n")
      if (all(adjmat == 0)) stop("fuck")
    }
  }
  ## Create a graph from the adjacency matrix
  G <- graph_from_adjacency_matrix(adjmat, mode = "directed", weighted = TRUE)
  G <- set_edge_attr(G, "width", value = 1 + log(edge_attr(G, "weight")))
  G <- set_vertex_attr(G, "size", value = 3)

  new("DeBruijn",
      G = G,
      adjmat = adjmat,
      motifs = motifs)
}

library(SVAlignR)

## single byte
data(longreads)
raw <- longreads$connection
alfa <- Cipher(raw)
coded <- encode(alfa, raw)
makeWords(coded, 3)
countWords(coded, 3)
countWords(coded, 3, alfa)

IT <- intToUtf8(L <- c(38:44, 46:57, 59:62, 64:126)) ## omit hyphen, colon, and question
JT <- paste(strsplit(IT, "|")[[1]], collapse = "-")
beta <- Cipher(JT)

twob <- encode(beta, JT)
double <- c(twob, twob)
countWords(double, 3)       # don't do this
countWords(double, 3, beta) # this is right

makeWords(double, 3)    # don't do this
makeWords(double, 3, 2) # this is right

m <- lapply(1:8, function(J) countWords(coded, J, alfa))
plotWords(3, m)

M <- lapply(1:8, function(J) countWords(double, J, beta))
plotWords(2, M)

setOldClass("igraph")
setClass("StringGraph",
         slots = c(name = "character",
                   edgelist = "matrix",
                   nodelist = "matrix",
                   graph = "igraph",
                   layout = "matrix"
                   )
)

setMethod("plot", "StringGraph", function(x,y, ...) {
  plot(x@graph, layout = x@layout, ...)
})


exportSG <- function(sg, outdir) {
  if (!inherits(sg, "StringGraph")) {
    stop("Wrong object type.\n")
  }
  fileedge <- file.path(outdir, paste(sg@name, "edgelist.csv", sep = "-"))
  filenode <- file.path(outdir, paste(sg@name, "nodelist.csv", sep = "-"))
  write.csv(sg@edgelist, file = fileedge, row.names = FALSE)
  temp <- data.frame(sg@nodelist, sg@layout)
  write.csv(temp, file = filenode, row.names = FALSE)
}

makeEdges <- function(A, B, debug = FALSE) {
  temp <- sapply(names(A), function(nm) {
    grepl(nm, names(B))
  })
  if (debug) cat(class(temp), "\n",  file = stderr())
  if (inherits(temp, "matrix")) {
    rownames(temp) <- names(B)
  } else {
    names(temp) <- names(B)
  }
  R <- expand.grid(names(B), names(A))
  R$Edge <- as.vector(temp)
  R[R$Edge,]
}

#################### MOTIFS ##########################

.makeEdgeListFromMotifs <- function(motifNodes) {
  motifCounts <- sapply(motifNodes, length)
  W <- which(motifCounts > 0)
  top <- max(W)
  EL <- list()
  for (i in 1:(top-1)) {
    A <- motifNodes[[i]]
    if (length(A) == 0) next # no nodes of this length
    for (j in (i+1):(top)) {
      B <- motifNodes[[j]]
      if (length(B) == 0) next # no nodes of this length
      EL <- rbind(EL, makeEdges(A, B))
    }
  }
  as.matrix(EL)
}
.makeNodeListFromMotifs <- function(motifNodes, crazyColors) {
  NL <- unique(unlist(sapply(motifNodes, names)))
  crazyColors <- createPalette(length(NL),
                               seedcolors = c("#ff0000", "#00ff00"), 
                               range = c(20, 80), M = 80000)
  names(crazyColors) <- NL
  stdcol <- Polychrome:::xform(crazyColors)
  luvmat <- as(hex2RGB(stdcol), "LUV")
  x <- luvmat@coords
  labelcols <- c("white", "black")[1 + 1*(x[,1] > 50)]
  names(labelcols) <- names(stdcol)
  nodeList <- data.frame(Names = NL, 
                         Type = "Motif",
                         Color = stdcol,
                         txtColor = labelcols)
  as.matrix(nodeList)
}

.makeLayoutFromMotifs <- function(NL, alfa, motifNodes) {
  XY <- data.frame(X = NA, Y = nchar(encode(alfa, NL))) # set Y values based on length.
  nMotifs <- sapply(motifNodes, length)
  for (I in which(nMotifs > 0)) {
    W <- which(XY$Y == I)
    top <- nMotifs[I]
    if (is.na(top)) stop("abruptly")
    XY$X[W] <- ((1:top)-top/2)*123
  }
  as.matrix(XY)
  rownames(XY) <- NL
  XY
}

.createGraph <- function(edgelist, nodelist) {
  X <- as.matrix(edgelist[, 1:2])
  M <- graph_from_edgelist(X, directed = TRUE)
  N <- vertex_attr(M, "name")
  if (length(N) < nrow(nodelist)) {
    warning("Some nodes have no edges.")
  }
  LL <- 1 + str_count(N, "\\-")
  M <- set_vertex_attr(M, "shape", value = "vrectangle")
  M <- set_vertex_attr(M, "color", value = nodelist[N, "Color"]) # get the colors
  M <- set_vertex_attr(M, "size", value = 4*LL + 3)
  M <- set_vertex_attr(M, "size2", value = 4)
  M <- set_edge_attr(M, "arrow.size", value = 0.3)
  M <- set_edge_attr(M, "weight", value = 10)
  M <- set_edge_attr(M, "color", value = "black")
  M
}

MotifGraph <- function(motifNodes, alfa, name = "motif") {
  edgelist <- .makeEdgeListFromMotifs(motifNodes)
  nodelist <- .makeNodeListFromMotifs(motifNodes, alfa)
  U <- unique(as.vector(as.matrix(edgelist[, 1:2])))
  W <- which(!(nodelist[, "Names"] %in% U))
  if (length(W) > 0) {
    nodelist <- nodelist[-W,]
  }
  layout <- .makeLayoutFromMotifs(nodelist[, "Names"], alfa, motifNodes)
  graph <- .createGraph(edgelist, nodelist)
  gname <- vertex_attr(graph, "name")
  layout <- as.matrix(layout[gname,])
  new("StringGraph",
      name = name,
      edgelist = edgelist,
      nodelist = nodelist,
      layout = layout,
      graph = graph)
}


################## DECOMP ############################

.makeEdgesFromDecomposition <- function(decomp) {
  SAP <- sapply(decomp, function(L) {
    a <- paste(L, collapse = "-")
    t(t(data.frame(motif = L, longread = a)))
  })
  EL2 <- do.call(rbind, SAP)  # edgelist
  EL2
}

.makeNodesFromDecomp <- function(edgelist, motifNodes) {
  temp <- unique(c(edgelist[,1], edgelist[,2]))
  nodeList2 <- data.frame(Names = temp,
                          Type = "LRBP",
                          Color = "#C0C0C0",
                          txtColor = "black")
  rownames(nodeList2) <- nodeList2[,"Names"]
  nodelist <- .makeNodeListFromMotifs(motifNodes)
  rownames(nodelist) <- nodelist[,"Names"]
  nodeList2[rownames(nodelist), ] <- nodelist
  nodeList2
}

.makeDecompLayout <- function(NL, alfa) {
  nuke <- nchar(encode(alfa, NL))
  XY <- data.frame(X = NA, Y = (-1)*nuke) # set Y based on length.
  nMotifs <- table(nuke)
  for (I in which(nMotifs > 0)) {
    W <- which(XY$Y == -I)
    top <- nMotifs[I]
    if (is.na(top)) stop("abruptly")
    XY$X[W] <- ((1:top)-top/2)*123
  }
  XY <- as.matrix(XY)
  XY[is.na(XY)] <- 0
  XY
}

DecompositionGraph <- function(decomp, alfa, motifNodes, name = "decomp") {
  edgelist <- .makeEdgesFromDecomposition(decomp)
  nodelist <- .makeNodesFromDecomp(edgelist, motifNodes)
  layout <- .makeDecompLayout(nodelist$Names, alfa)
  rownames(layout) <- nodelist$Names
  graph <- .createGraph(edgelist, nodelist)
  VA <- vertex_attr(graph, "name")
  layout <- layout[VA,]
  new("StringGraph",
      name = name,
      edgelist = as.matrix(edgelist),
      nodelist = as.matrix(nodelist),
      layout = layout,
      graph = graph)
}

---
title: "SVAlignR and Virus-Associated Cancer"
author: "Kevin R. Coombes, Keiko Akagi, Davied Symer, and Maura Gillison"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_document:
    theme: journal
    highlight: kate
  toc: true
vignette: >
  %\VignetteIndexEntry{SVAlignR and Virus-Associated Cancer}
  %\VignetteKeywords{OOMPA,CytoGPS,karyotypes,idiograms,graphics}
  %\VignetteDepends{SVAlignR}
  %\VignettePackage{SVAlignR}
  %\VignetteEngine{knitr::rmarkdown}
---


```{r opts, echo=FALSE, results="hide"}
knitr::opts_chunk$set(fig.width=8, fig.height=5)
options(width = 88)
```
```{r mycss, results="asis", echo=FALSE}
cat('
<style type="text/css">
.figure { text-align: center; }
.caption { font-weight: bold; }
</style>
')
```
# Introduction
Some cancers are associated with the presence of viruses that disrupt chromosomal DNA.
Among the best known examples are cervical cancers and head-and-neck cancers that are
associated with human papilloma virus (HPV). Using whole genome short-read sequencing,
it is possible to identify breakpoints, including those where viral DNA is merged with
human DNA. Each breakpoint defines a  local instance of "structural variation" (SV) where
segments of chromosomes are joined together in non-normal ways. It remains an open
question whether those fragments are fully integrated into
a human chromosome or just exist as circular extrachromosomal DNA. Using newer long-read
sequencing technology, one can obtain information about the order of multiple breakpoints
in the original DNA molecules in the tumor sample. In this vignette, we describe SVAlignR,
an R package intended to use these long-read sequences to reconstruct the original molecules.

# Getting Started
We begin by loading the `SVAlignR` package, and a sample data set.
```{r Setup}
library(SVAlignR)
data(longreads)
head(longreads[, 1:3])
```
Each of the  `r nrow(longreads)` rows in this spreadsheet represents a different Oxford
Nanopore long read.  The `qname` column is a unique identifier; the `qlen` column contains
the length of the read in bytes; and the `connections` column lists the breakpoints that 
the read contains, each of which has been assigned a numeric identifier. We are interested
in this column.  We have two main goals:

1. Assemble the long-reads (which should be thought of as fragmentary) into complete molecules.
2. Start constructing a phylogenetic tree showing the evolution of these HPV-human combining
molecules.

As with many sequence alignment problems, we start by removing any duplicate "reads"; here
that means duplication of the break-point sequence summaries of the actual reads.
```{r LR}
LR0 <- longreads$connection
names(LR0) <- rownames(longreads)
LR <- LR0[!duplicated(LR0)]
weights <- sapply(LR, function(lr) sum(LR0 == lr))
length(LR) # expect 158
```

# Clustering Sequences
We want to cluster the break-point sequences.

## Recoding
In order to make use of existing algorithms,
we need to convert the numeric identifiers into single characters as part of an "alphabet". 
The `Cipher` class takes in a character vector containing the "words" (that is, our
break-point sequences) that need to be rewritten. By default, it assumes (as in our example)
that "letters" are separated by hyphens, identifies the unique symbols, and creates a tool
to convert back-and-forth from one "alphabet" to another.
```{r unique.symbols}
alphabet <- Cipher(LR)
alphabet
```

Now we are able to convert the original sequences of numbers into words in our specialized
alphabet.
```{r pseudo}
head(LR)
pseudo <- encode(alphabet, LR)
head(pseudo)
```

## Clusters
The recoding step is already included in the `SequenceCluster` constructor and will happen automatically.
So, to cluster the long-read break point sequences, we just need to call the constructor on the
(named and de-duplicated) sequences. The core algorithm is the classic
Needelman-Wunsch global alignment algorithm, as implemented in the `NameNeedle` R package. As
used in the `SVAlignR` package, it computes an alignment score between any two sequences based
on a match score of 2, a mismatch penalty of -6, and a gap penalty of -2.  We fill a square
matrix with scores for every pair (I, J) of sequences. The resulting scores can be either
positive or negative. For each row X, we rescale into the interval [0,1] by computing
$$Y_{IJ} = 1 - \frac{X_{IJ} - min(X_{I.})}{max(X_{I.})-min(X_{I.})}.$$
Since the scores need not be symmetric, we force symmetry by averaging the scaled score matrix and
its transpose.

We then perform hierarchical clustering using the symmetrized, rescaled score matrix to define
distances. We cut the dendrogam to cluster the sequences into 20 groups. The number 20 is
arbitrary. In our data set, it should yield about 8 long-read break-point sequences per cluster.
For later steps, we also want to make sure that each cluster contains at leeast 2 sequences.
```{r compute.distances}
seqclust <- SequenceCluster(LR0, NC = 15)
table(seqclust@clusters)
```
Now we see why 20 clusters is appropriate. Each cluster is a relatively small size, but they
all contain more than one sequence so that aligning those sequences will be meaningful.
**Figure 1** displays the resulting dendrogram, with each of the 20 clusters shown in a
different color.
```{r fig01, fig.cap = "Figure 1: Hierarchical clustering of break point sequences.", fig.width = 12}
plot(seqclust)
```

**Figure 2**  contains a simplified view of the same dendrogram, cutting off the clutter
below the major splits into clusters.
```{r fig02, fig.width=8, fig.cap = "Figure 2: Uncluttered clusters."}
plot(seqclust, type = "clipped")
```

**Figure 3** displays an alternate view of the dendrogram, emphasizing the relationships
between clusters. 
```{r fig03, fig.cap = "Figure 3: Unrooted phylogeny tree.", fig.width=9, fig.height=9}
plot(seqclust, type = "unrooted")
```

Both **Figure 2** and, more strongly, **Figure 3** indicate that there are approximately four
major clusters.

We can also show the dendrogram attached to a heatmap containing the distance matrix.

```{r fig04, fig.cap = "Figure 4: Heatmap of the distance matrix.", fig.width=9, fig.height=9}
heat(seqclust)
```

# Aligning Clustered Sequences
Each of the clusters uses a small enough set of symbols that we can safely rewrite them as
though they are sequences of amino acids. We then use the `align` function from the `SVAlignR`
package to perform multiple sequence alignments, using the "ClustalW" algorithm on each cluster.
Because the components are not amino acids, however, we do not need to use one of the PAM or
BLOSUM matrices. Instead, we define a substitution matrix that gives the same match and mismatch
scores to each pair of symbols.

```{r do.align}
if (!requireNamespace("msa", quietly = TRUE)) {
  stop("Cluster alignment only works if the `msa` package is available.")
}
MYSUB <- makeSubsMatrix(match = 2, mismatch = -6)
consensus <- alignAllClusters(seqclust, mysub = MYSUB, gapO = 2, gapE = 0.1)
class(consensus)
length(consensus)
```

**Figure 5** contains an example of a multiple sequence alignment.
```{r fig05 , fig.cap="Figure 5: Example alignment of cluster 15."}
opar <- par(mai = c(1.02, 0.82, 0.82, 0.82))
image(consensus[[15]])
par(opar)
rm(opar)
```

## Alignments By Cluster
**Figure 6** shows all 20 alignments. 
```{r fig06, fig.cap="Figure 6: Multiple sequence alignments of clusters.", fig.width=20, fig.height=20}
opar <- par(mfrow = c(4,3))
for (K in 1:length(consensus)) {
  image(consensus[[K]], col = SVAlignR:::myColorSet[K])
}
par(opar)
```

## Consensus Sequences

```{r conseq, fig.cap="Alignment of all consensus sequences.", echo=FALSE, eval = FALSE}
allmodcons <- sapply(consensus, function(X) X@consensus)
allmodcons <- gsub("\\?", "q", allmodcons) # make question mark a letter
allmodcons <- gsub("\\:", "z", allmodcons) # make rare colon a letter
while (length(grep("^[qz]", allmodcons) > 0)) {
  allmodcons <- sub("^q-", "", allmodcons)
  allmodcons <- sub("^z-", "", allmodcons)
}
while (length(grep("[qz]$", allmodcons) > 0)) {
  allmodcons <- sub("-q$", "", allmodcons)
  allmodcons <- sub("-z$", "", allmodcons)
}
names(allmodcons) <- paste("C", c(rep(0,9), rep("", 11)), 1:20, sep = "")

newsub <- makeSubsMatrix(match = 2, mismatch = -6)
ab <- alignCluster(allmodcons, mysub = newsub, gapO = 0.1, gapE = 0.2)
image(ab)

```


```{r fig07, fig.cap = "Figure 7: Unrooted phylogeny tree with consensus sequences.", fig.width=9, fig.height=9}
library(dendextend)
myColorSet <- SVAlignR:::myColorSet
NC <- seqclust@NC
mycut <- mean(rev(seqclust@hc$height)[NC + -1:0])
hcd <- as.dendrogram(seqclust@hc)
dend2 <- cut(hcd, h = mycut)
DU <- dend2$upper
blocks <- sapply(consensus, function(X) {
  X@consensus
})
labels(DU) <- paste("   [", 1:NC, "]  ",  blocks, "   ", sep = "")

opar <- par(bg = "gray90")
plot(ape::as.phylo(DU), type = "unrooted", rotate.tree = -20,
     edge.color = "black", edge.width = 2,
     lab4ut = "axial", 
     tip.color = myColorSet[1:NC], cex = 1.2)
par(opar)

```

```{r makepngs, eval = FALSE, echo=FALSE, results = "hide"}
resn <- 300
for (K in 1:NK) {
  temp <- consensus[[K]]
  png(file = file.path(paths$figs, paste("cons-", K, ".png", sep = "")),
      width = 6*resn, height = 4*resn, res = resn, bg = "white")
  showme(temp, col = SVAlignR:::myColorSet[K], cex = 0.5)
  dev.off()
}
```

# De Bruijn Graphs
We also provide the ability to compute de Bruijn graphs from any set of sequences. These graphs are
frequently used for "de novo" sequencing, and thus can provide an alternative source of information
about "consensus sequence" alignments. In order to work with graphs in general, you should load the
`igraph` package. 

```{r deBruijn}
library("igraph")
DB <- deBruijn(seqclust@rawSequences, 4)
G <- DB@G
loci <- layout_with_fr(G)
plot(G, layout = loci)


count_components(G)
comp <- components(G)
sort(comp$csize)

GC <- largest_component(G)
gloc <- layout_with_gem(GC)
plot(GC, layout = gloc)

sort(table(E(GC)$weight))
GE <- delete_edges(GC, E(GC)[E(GC)$weight  < 6])
GE <- largest_component(GE)
#GE <- set_edge_attr(GE, "width", value = sqrt(edge_attr(G, "weight")))
gloc <- layout_with_fr(GE)
plot(GE, layout = gloc)
```

```{r debruijn sets}
dbset <- lapply(1:seqclust@NC, function(J) {
  rs <- seqclust@rawSequences[seqclust@clusters == J]
  db <- deBruijn(rs, 4)
}) 
L <- length(dbset)
for (I in 1:L) {
  G <- dbset[[I]]@G
  lyt <- layout_with_fr(G)
  plot(G, layout =lyt, main = paste("Cluster", I))
}

```

# Appendix
```{r si}
sessionInfo()
```

\name{AlignedCluster-class}
\docType{class}
\alias{AlignedCluster-class}
\alias{alignCluster}
\alias{alignAllClusters}
\alias{makeSubsMatrix}
\alias{image,AlignedCluster-method}

\title{Class \code{"AlignedCluster"}}
\description{
The \code{AlignedCluster} class is used to align a set of clustered
sequences. The \code{alignClusters} function creates a new object of the
\code{AlignedCluster} class. The \code{alignAllClusters} function takes
a \code{SequenceCluster} object and returns a list of
\code{AlignedCluster} objects. Clustering is performed using the
ClustalW algorithm. The associated class and functions take care of
encoding and decoding sequences into a form that can be used by the
implementation of ClustalW in the \code{msa} package.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{alignCluster} or
  \code{alignAllCluster} functions. You typically pass in a character
  vector of sequences that have already been found to form a cluster.
}
\usage{
alignCluster(sequences, mysub = NULL, gapO = 10, gapE = 0.2)
alignAllClusters(sc, mysub = NULL, gapO = 10, gapE = 0.2)
makeSubsMatrix(match = 5, mismatch = -2)
\S4method{image}{AlignedCluster}(x, counts = TRUE, col = "black", cex = 1, main = "", ...)
}
\arguments{
  \item{sequences}{A character vector that contains all sequences
    to be aligned.}
  \item{mysub}{A square (usually symmetric) substitution matrix.}
  \item{gapO}{A numeric value defining the penalty for opening a gap.}
  \item{gapE}{A numeric value defining the penalty for extending a gap.}
  \item{sc}{An object of the \code{SequenceCluster} class.}
  \item{match}{A numeric value defining the reward for matching symbols
    from two sequences.}
  \item{mismatch}{A numeric value defining the penalty for mismatching
    symbols from two sequences.}
  \item{x}{An object of the \code{AlignedCluster} class.}
  \item{col}{A character setting the color of annotations in the image.}
  \item{counts}{A logical value; whether to display the repeat counts of
    sequences on the right side of the image.}
  \item{main}{Character; the plot title.}
  \item{cex}{Numeric; size of the text inside the image of the alignment
    matrix.}
  \item{\dots}{Extra arguments for generic or plotting routines.}
}
\section{Slots}{
  \describe{
      \item{\code{alignment}:}{A matrix of aligned sequences; rows are
	sequences and columns are aligned positions..}
      \item{weights}{A numeric vactor; the numbof times each unique raw
	sequence occurs.}
      \item{\code{consensus}:}{A character vector; the consensus
	sequence of a successful alignment.}
  }
}
\section{Details}{
  Alignment is performed using the implementation of the \code{ClustalW}
  algorithm provided by the \code{msa} package. The existing code to align
  amino-acid protein sequences is used by converting the current alphabet
  to one that limits its use to the known amino acids. The decision to
  ue this method introduces a limitation: we are unable to align any set
  of seqeunces that use more than 25 distinct symbols. Attempting such
  an alignment will result in the \code{alignCluster} function returning
  a \code{NULL} value, which is passed on as one of the list items from
  \code{alignAllClusters}.
}
\value{
The \code{alignCluster} function returns a new object of the \code{AlignedCluster}
class. The \code{alignAllClusters} function returns a list of
\code{AlignedCluster} objects. The \code{makeSubMatrix} function returns
a symmetric substitution matrix.
}
\note{
  These functions will only work if the \code{ms} package is
  installed. At the time of writing, CRAN does not install \code{msa}
  because of the way that \code{msa} uses the \code{OpenMP}
  protocol. So, \code{SVAlignR} only "Suggests" using the package and
  does not include it in the list of "Imports". Thus, to obtain this
  functionality, you must manually install \code{msa} from the
  yourself using the \code{BiocManager::install} function from
  BioConductor. 
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
data(longreads)
seqs <- longreads$connection[1:15]
pad <- c(rep("0", 9), rep("", 6))
names(seqs) <- paste("LR", pad, 1:length(seqs), sep = "")
seqs <- seqs[!duplicated(seqs)]
mysub <- makeSubsMatrix(match = 2, mismatch = -6)
if (!requireNamespace("msa", quietly = TRUE)) {
  warning("Cluster alignment is only available if the 'msa' package is installed.\n")
} else {
  ab <- alignCluster(seqs, mysub)
  image(ab)
}
}
\keyword{ cluster }

\name{StringGraph-class}
\docType{class}
\alias{StringGraph}
\alias{StringGraph-class}
\alias{plot,StringGraph,ANY-method}
\alias{MotifGraph}
\alias{DecompositionGraph}
\alias{exportSG}

\title{Class \code{"StringGraph"}}
\description{
The \code{StringGraph} class is used to represent graphs that arise from
strings reprsenting long-read breakpoint sequences. The basic examples
are: (1) "Motif Graphs" where the edges are subtring relations, and (2)
"Decomposition Graphs" where the edges are restricted subtring relations
that decompose a long read.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{MotifGraph} or
  \code{DecompositionGraph} constructor. You typically pass in a
  "motifNodes" object, which is a list of sequence-strings separated by
  length, along with some auxiliary information.
}
\usage{
MotifGraph(motifNodes, alfa, name = "motif")
DecompositionGraph(decomp, alfa, motifNodes, name = "decomp")
exportSG(sg, outdir)
\S4method{plot}{StringGraph,ANY}(x, y, \dots)
}
\arguments{
  \item{motifNodes}{A list of node names and counts, separated by
    length. In particular, \code{motifNodes[[L]]} should contain the
    nodes of length L.}
  \item{alfa}{A \code{Cipher} object.}
  \item{name}{A character vector of length one.}
  \item{decomp}{A decomposition object; see details.}
  \item{sg}{An object of the \code{StringGraph} class.}
  \item{outdir}{A chara cter string, the name of the output directory.}
  \item{x}{An object of the \code{StringGraph} class.}
  \item{y}{Anything; it is ignored.}
  \item{\dots}{Extra graphical parameters.}
}
\section{Slots}{
  \describe{
      \item{\code{name}:}{A character vector of length one.}
      \item{\code{edgelist}:}{A matrix representing a graph as a list of edges.}
      \item{\code{nodelist}:}{A matrix representing the nodes of the
	graph, along with their properties.}
      \item{\code{graph}:}{An \code{igraph} object.}
      \item{\code{layout}:}{A matrix containing x-y locations for the nodes.}
  }
}
\value{
The \code{MotifGraph} and \code{DecompositionGraph} functions return a
new object of the \code{StringGraph} class. The \code{plot} method and
\code{exportSG} functions return nothing and are called for their side
effects.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\note{
Attempting to manipulate a \code{StringGraph} object using text containing
NAs, missing values, or previously unknown symbols will result in an error.
}
\keyword{ math }

\name{Circular DNA Evidence}
\alias{getEvidence}
\alias{showCoverage}

\title{Support and Evidence for Circular DNA}
\description{
  Tools to assess the evidence supporting the idea that a long-read
  breakpoint sequence actually represents a piece of extrachromosomal
  circular DNA.
}
\usage{
getEvidence(cycles,  seqs, alfa, totalOnly = TRUE)
showCoverage(ID, evidence, alfa, colsams, locsams = NULL)
}
\arguments{
  \item{cycles}{A character vector containg the breakpoint strings of
    the potential circles to be evaluated.}
  \item{seqs}{A chaarcter vector continaing the breakpoint sequences of
    the long read seqeunces to be used for support of circles.}
  \item{alfa}{An object of the \code{\link{Cipher}} class.}
  \item{totalOnly}{A logical value; only return cycles that are totally
    supported?}
  \item{ID}{An integer index into the "coverage" list indicating which
    cycle to display.}
  \item{evidence}{A list as output by the \code{getEvidence} function.}
  \item{colsams}{A character vector indicating which colors to use for
    supporting long reads. If missing, all long reads are colored grey.}
  \item{locsams}{A character vector indicating which samples contained
    the long reads.}
}
\details{
TDA
}
\value{
  The \code{getEvidence} function returns a list with three entries:
  \itemize{
    \item{\code{coverage} is itself a list with one entry for each
    cycle. These entries contain \code{N} (an integer number of
    supporting reads), \code{RDS} (a character string with long-read
    identifiers seaparated by semicolons), \code{TOT} (an integer, the
    minimum support level for the nodes/brealpoints in the cycle),
    \code{WTOT} (an integer, the minimum weighted count of support
    across nodes), \code{COV} (an intege vector showing the number of
    reads supporting each individual node, and \code{WCOV} the weighted
    version of \code{COV}).}
    \item{\code{circles} is a data frame with one row for each cycles and
    columns \code{N}, \code{RDS}, \code{TOT}, \code{WTOT}, and
    \code{cycle}. If \code{totalOnly} was \code{TRUE}, only totally
    supported cycles (those with \code{TOT>0}) are returned.}
    \item{\code{vspan} a character vector containng the strings
    describing an expanded set of potential cycles.}
  }
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  TDA
}
\seealso{
  TDA
}
\examples{
getEvidence
}
\keyword{hplot}
\keyword{models}

\name{Cipher-class}
\docType{class}
\alias{Cipher}
\alias{Cipher-class}
\alias{encode}
\alias{decode}

\title{Class \code{"Cipher"}}
\description{
The \code{Cipher} class is used to change between different alphabets
(and so behaves as a simple substitution cipher). The \code{Cipher} function
creates a new object of the \code{Cipher} class.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{Cipher} constructor. You
  typically pass in a character vector of "words" that contain all the
  symbols that are contained in the text to be translated (i.e., encoded
  and decoded) between languages. A standard target alphabet is created
  along with forward and reverse transliteration rules.
}
\usage{
Cipher(sampleText, split = "-", extras = c("-" = ":", "?" = "?"))
encode(cipher, text)
decode(cipher, text)
}
\arguments{
  \item{sampleText}{A character vector that contains all symbols you
    want to be able to transliterate. Duplicate symbols are
    automatically removed.}
  \item{split}{A single character used to split words into
    symbols. Defaults to a hyphen for our applications.}
  \item{extras}{Additional characters to be added for reverse
    tranlsiteration, since they may appear as the results of
    alignments in consensus sequences.}
  \item{cipher}{An object of the \code{Cipher} class.}
  \item{text}{A character vector of words to be transliterated.}
}
\section{Slots}{
  \describe{
      \item{\code{forward}:}{A named character vector.}
      \item{\code{reverse}:}{A named character vector.}
      \item{\code{bytes}:}{The number of bytes used to encode each
	'character' in the input test. Text with more than 72 unique
	characters use a two-byte encoding, which is enough for
	languages with up to 26*72 = 1872 characters.} 
  }
}
\value{
The \code{Cipher} function returns a new object of the \code{Cipher}
class. The \code{encode} and \code{decode} functions return character
vectors that are the same size as their input \code{text} parameters.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\note{
Attempting to manipulate a \code{Cipher} object using text containing
NAs, missing values, or previously unknown symbols will result in an error.
}
\examples{
motif <- "0-50-74-0-50-74-25-26-35"
alfa <- Cipher(motif)
alfa
en <-encode(alfa, motif)
en
de <- decode(alfa, en)
de
}
\keyword{ math }

\name{SVAlignR-data}
\alias{SVAlignR-data}
\alias{longreads}
\alias{breakpoints}
\docType{data}
\title{SVAlignR Sample Data}
\description{
  These data sets contain binary versions of data describing breakpoints
  and long read sequences from an HPV-positive head-and-neck cancer sample.
}
\usage{
data("longreads")
}
\format{
  \describe{
    \item{\code{longreads}}{A data frame with 197 rows and 5 columns.
      Each row represents a single Oxford Nanopore long read from a study
      of a cell line from an HPV-positive head-and-neck squamous cell
      tumor. The five columns contain (i) a unique identifier of each
      long read, (ii) the length of the read, in bytes, (iii) the
      ordered sequence of break points, represented as a hyphen
      separated list of numeric identifiers, (iv) manually estimated
      natural groups of reads, and (v) a manually curated indication of
      whether certain long reads should be omitted from the analysis.
    }
    \item{\code{breakpoints}}{A data frame with 82 rows and 11 columns.
      Each row represents a single breakpoint from a study of a cell line
      from an HPV-positive head-and-neck squamous cell tumor. The columns
      contain (1) a unique identifier that is used in the long read
      connections, (2-4) a description of the chromosomal segment to the
      left of the breakpoint, (5-7) a description of the chromosomal
      segment to the right of the breakpoint, (8-9) the orientation of the
      two chromosomal segments, (10) a shorthand description of the
      breakpoint with the segment names separated by a vertical bar and
      negative strands contained in parentheses, and (11) a shorthand
      representation of the reverse orientation of the breakpoint.
    }
  }
}
\source{
  Long read (Oxford Nanopore) sequencing was performed on samples
  prepared at the laboratory of Maura Gillison and David   Symer.
  Characterization of long reads as a sequence of well-defined
  break points was performed by Keiko Akagi.
}
\author{
  Kevin R. Coombes <krc@silicovore.com>
}
\examples{
data(longreads)
head(longreads)

alphabet <- Cipher(longreads$connection)
en <- encode(alphabet, "0-50-74-0-50-74-35")
en
decode(alphabet, en)
}
\keyword{datasets}

\name{Words}
\docType{class}
\alias{makeWords}
\alias{countWords}
\alias{plotWords}

\title{Class \code{"Words"}}
\description{
Provides the ability to find, count, and plot words of specific length
in collections of strings in any sequence language.
}
\usage{
makeWords(opstrings, K, nb = 1)
countWords(opstrings, K, alpha = NULL)
plotWords(K, m)
}
\arguments{
  \item{opstrings}{A character vector containing a set of words that
    have been encoded into an alphabet where each character uses the same
    number of bytes in the encoding.}
  \item{K}{An integer; the length of the words of interest.}
  \item{nb}{An integer; the number of bytes used to encode each character.}
  \item{alpha}{A \code{Cipher} object, used to decode the word-strings.}
  \item{m}{A list of word-counts produced by the \code{makeWords} function.}
}
\details{
  For constructing motifs, or for producing De Bruijn graphs, we need to
  be able to decompose a set of input strings into "words" of a fixed
  length. In our application, the words are derived from long-read
  sequences that cross multiple breakpoints. Each breakpoint is given a
  unique name/label, thatwhich can be of arbirtrary length in order to be
  maningful to the researchers. Using the \code{\link{Cipher}} class, we
  encode the breakpoint names into character strings of the same
  size. (In the original version of this package, we used single
  characters. That approach eventually proved to be inadequate when we
  looked at long-read data from samples with a very large number of
  breakpoints. We then extended the package to work with two-byte
  codes. This solution may eventually be extended to even longer coding
  sequences.)

  The \code{makeWords} and \code{countWords} functions take as inputs a
  vector of character strings (typically describing long-read
  sequences) that have already been encoded into fixed-byte-length
  characters.  They then find all words in those strings of a given
  fixed length. They only differ in the form of their output. The former
  function returns the word counts in their encoded form; the latter
  decodes them back to the original names (as long as you provide the
  optional appropriate Cipher argument).

  The \code{plotWords} function gives a visible representaiton of words
  of length \code{K} sorted by their frequency. The x-axis contains the
  sorted word list; the y-axis is the frequency. The idea is that one
  can quickly figure out which words are most common in the input "text".
}
\value{
  The \code{makeWords} function returns a table of words (of length
  \code{K}) along with the counts of the number of times each one was
  seen in the input strings. The \code{countWords} function returns the
  same table, but with the words decoded back to the original language.
  The \code{plotWords} function returns a vector of the word counts for
  all words of length \code{K} in the list \code{m}.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
data(longreads)             # read sample data
raw <- longreads$connection # get the raw strings
alfa <- Cipher(raw)         # make a translation cipher
coded <- encode(alfa, raw)  # encode all the input strings
makeWords(coded, 3)
countWords(coded, 3, alfa)
m <- lapply(1:8, function(J) countWords(coded, J, alfa))
plotWords(3, m)
}
\keyword{ manip }

\name{Breakpoints-class}
\docType{class}
\alias{Breakpoints}
\alias{Breakpoints-class}
\alias{plot,Breakpoints,missing-method}

\title{Class \code{"Breakpoints"}}
\description{
  Classes for working with collections of breakpoints.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{Breakpoints} constructor. You
  typically pass in a data frame containing columns with the name/id of
  the breakpoint, and their chromosome name, start, and stop positions for
  each side of the break.
}
\usage{
Breakpoints(working)
\S4method{plot}{Breakpoints,missing}(x, y, colset, \dots)
}
\arguments{
  \item{working}{A data frame containing the locations of break
    points. These should be seven consecutive columns, starting with the
    break point id followed by three columns each (chromosome, start,
    stop) for each side of the break point.}
  \item{x}{An object of the \code{Breakpoints} class.}
  \item{y}{Anything; it is ignored.}
  \item{colset}{A character vector of color specifications.}
  \item{\dots}{Extra graphical parameters.}
}
\section{Slots}{
  \describe{
      \item{\code{relLocation}:}{A numeric vector giving relative
	coordinates (in the unit interval) of the breakpoints along a
	chromosome, with first and last break points mapped to 0 and 1.}
      \item{\code{labels}:}{A character vector containing the names
	of the chromosomes.}
      \item{\code{ypos}:}{A numeric vector indicating the chromosomes
	involved in the full set of break points.}
      \item{\code{spread}:}{How far the display of different
	chromosomes should be spread apart on the y-axis.}
      \item{\code{id}:}{The character vector of break point names.}
  }
}
\value{
The \code{Breakpoints} constructor returna a newly created object of the
\code{Breakpoints} class. The \code{plot} method invisible returns its
first argument.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\keyword{ manip }

\name{DeBruijn-class}
\docType{class}
\alias{deBruijn}
\alias{DeBruijn-class}

\title{Class \code{"DeBruijn"}}
\description{
  Classes for contructing de Bruijn graphs from collections of long read
  sequences mappe over brakpoints.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{deBruijn} constructor.
}
\usage{
deBruijn(rawseq, M)
}
\arguments{
  \item{rawseq}{A character vecvtor of the long read sequences,
    expressed as hyphen-separated breakpoint ids.}
  \item{M}{An integer; the length of the motifs/words to be used in
    constructing the graph.}
}
\section{Slots}{
  \describe{
      \item{\code{G}:}{An object of hhe \code{\link[igraph]{igraph}} class}
      \item{\code{adjmat}:}{An adjacency matrix.}
      \item{\code{motifs}:}{A table of motifs/words.}
  }
}
\value{
The \code{deBruijn} constructor returns a newly created object of the
\code{DeBruijn} class.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\keyword{ manip }

\name{SequenceCluster-class}
\docType{class}
\alias{SequenceCluster}
\alias{updateClusters}
\alias{heat}
\alias{SequenceCluster-class}
\alias{plot,SequenceCluster,missing-method}

\title{Class \code{"SequenceCluster"}}
\description{
The \code{SequenceCluster} class is used to cluster sequences of "words"
from an arbitrarily long alphabet. The \code{SequenceCluster} function
returns a new object of the \code{SequenceCluster} class.
}
\section{Objects from the Class}{
  Objects should be defined using the \code{SequenceCluster} constructor. You
  typically pass in a character vector of "words"  to be clustered.
}
\usage{
SequenceCluster(rawseq, method = c("needelman", "levenshtein"), NC = 5)
\S4method{plot}{SequenceCluster,missing}(x, type = "rooted", main = "Colored Clusters", \dots)
updateClusters(sc, NC)
heat(x, \dots)
}
\arguments{
  \item{rawseq}{A character vector that contains all words or "sequences"
    to be clustered.}
  \item{method}{The algorithm to use to compute distances between
    sequences. The choices are "levenshstein", which uses the
    Levenshtein edit distance, or "needelman", which uses the
    Needelman-Wunsch global alignment algorithm.}
  \item{x}{An object of the \code{SequenceCluster} class.}
  \item{sc}{An object of the \code{SequenceCluster} class.}
  \item{NC}{An integer; the number of clusters to cut from the dendrogram.}
  \item{type}{A character strnig; the type of plot to make. Valid types
    are "rooted", "clipped", or "unrooted".}
  \item{main}{Character; the plot title.}
  \item{\dots}{extra arguments for generic or plotting routines}
}
\section{Slots}{
  \describe{
      \item{\code{method}:}{A character vector describing which
	algorithm was used.}
      \item{rawSequences}{A character vector that contains the input words or "sequences"
	tthat were clustered.}
      \item{weights}{A numeric vactor; the numbof times each unique raw
	sequence occurs.}
      \item{\code{distance}:}{A \code{dist} object.}
      \item{\code{hc}:}{An \code{hclust} object.}
      \item{\code{NC}:}{An integer; the number of clusters cut from the dendrogram.}
      \item{\code{clusters}:}{An integer vector containing cluster assignments.}
  }
}
\value{
The \code{SequenceCluster} function returns a new object of the \code{SequenceCluster}
class.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
data(longreads)
sequences <- longreads$connection[1:30]        # named character vector
sequences <- sequences[!duplicated(sequences)] # dedup
sc <- SequenceCluster(sequences)               # cluster
plot(sc)                                    # visualize
sc <- updateClusters(sc, NC = 7)
plot(sc, type = "unrooted")
}
\keyword{ cluster }
